/* Example taken from Google Calendar API JavaScript Quickstart https://developers.google.com/google-apps/calendar/quickstart/js */

{
    // Your Client ID can be retrieved from your project in the Google
    // Developer Console, https://console.developers.google.com
    var CLIENT_ID = "<YOUR_CLIENT_ID>";

    var SCOPES = ["https://www.googleapis.com/auth/calendar.readonly"];

    /**
     * Check if current user has authorized this application.
     */
    function checkAuth() {
        gapi.auth.authorize(
            {
                "client_id": CLIENT_ID,
                "scope": SCOPES.join(" "),
                "immediate": true,
            },
            handleAuthResult,
        );
    }

    /**
     * Handle response from authorization server.
     *
     * @param {Object} authResult Authorization result.
     */
    function handleAuthResult(authResult: GoogleApiOAuth2TokenObject) {
        var authorizeDiv = document.getElementById("authorize-div")!;
        if (authResult && !authResult.error) {
            // Hide auth UI, then load client library.
            authorizeDiv.style.display = "none";
            loadCalendarApi();
        } else {
            // Show auth UI, allowing the user to initiate authorization by
            // clicking authorize button.
            authorizeDiv.style.display = "inline";
        }
    }

    /**
     * Initiate auth flow in response to user clicking authorize button.
     *
     * @param {Event} event Button click event.
     */
    function handleAuthClick(event: MouseEvent) {
        gapi.auth.authorize(
            { client_id: CLIENT_ID, scope: SCOPES, immediate: false },
            handleAuthResult,
        );
        return false;
    }

    /**
     * Load Google Calendar client library. List upcoming events
     * once client library is loaded.
     */
    function loadCalendarApi() {
        gapi.client.load("calendar", "v3", listUpcomingEvents);
    }

    /**
     * Print the summary and start datetime/date of the next ten events in
     * the authorized user's calendar. If no events are found an
     * appropriate message is printed.
     */
    function listUpcomingEvents() {
        var request = gapi.client.calendar.events.list({
            "calendarId": "primary",
            "timeMin": (new Date()).toISOString(),
            "showDeleted": false,
            "singleEvents": true,
            "maxResults": 10,
            "orderBy": "startTime",
        });

        request.execute(function(resp) {
            var events = resp.items;
            appendPre("Upcoming events:");

            if (events.length > 0) {
                for (let i = 0; i < events.length; i++) {
                    var event = events[i];
                    var when = event.start.dateTime;
                    if (!when) {
                        when = event.start.date;
                    }
                    appendPre(event.summary + " (" + when + ")");
                }
            } else {
                appendPre("No upcoming events found.");
            }
        });
    }

    /**
     * Append a pre element to the body containing the given message
     * as its text node.
     *
     * @param {string} message Text to be placed in pre element.
     */
    function appendPre(message: string) {
        var pre = document.getElementById("output")!;
        var textContent = document.createTextNode(message + "\n");
        pre.appendChild(textContent);
    }
}

/* Example taken from https://developers.google.com/google-apps/calendar/v3/reference/events/insert#examples */

{
    // Refer to the JavaScript quickstart on how to setup the environment:
    // https://developers.google.com/google-apps/calendar/quickstart/js
    // Change the scope to 'https://www.googleapis.com/auth/calendar' and delete any
    // stored credentials.

    const event = {
        "summary": "Google I/O 2015",
        "location": "800 Howard St., San Francisco, CA 94103",
        "description": "A chance to hear more about Google's developer products.",
        "start": {
            "dateTime": "2015-05-28T09:00:00-07:00",
            "timeZone": "America/Los_Angeles",
        },
        "end": {
            "dateTime": "2015-05-28T17:00:00-07:00",
            "timeZone": "America/Los_Angeles",
        },
        "recurrence": [
            "RRULE:FREQ=DAILY;COUNT=2",
        ],
        "attendees": [
            { "email": "lpage@example.com" },
            { "email": "sbrin@example.com" },
        ],
        "reminders": {
            "useDefault": false,
            "overrides": [
                { "method": "email", "minutes": 24 * 60 },
                { "method": "popup", "minutes": 10 },
            ],
        },
    };

    var request = gapi.client.calendar.events.insert({
        "calendarId": "primary",
        "resource": event,
    });

    request.execute(function(event) {
        appendPre("Event created: " + event.htmlLink);
    });

    function deleteEvent(calendarEventId: any) {
        var request = gapi.client.calendar.events.delete({
            calendarId: "primary",
            eventId: calendarEventId,
            sendNotifications: true,
        });

        request.then(
            (response) => {
                appendPre("Successfully deleted event " + calendarEventId);
            },
            (error) => {
                appendPre("Unable to delete event from your calendar" + calendarEventId);
            },
        );
    }
}
